import pytest

import humps


def test_converting_strings():
    assert humps.camelize("jack_in_the_box") == "jackInTheBox"
    assert humps.decamelize("rubyTuesdays") == "ruby_tuesdays"
    assert humps.depascalize("UnosPizza") == "unos_pizza"
    assert humps.pascalize("red_robin") == "RedRobin"


@pytest.mark.parametrize(
    "input_str, expected_output",
    [
        ("PERatio", "pe_ratio"),
        ("HTTPResponse", "http_response"),
        ("_HTTPResponse", "_http_response"),
        ("_HTTPResponse__", "_http_response__"),
        ("BIP73", "BIP73"),
        ("BIP72b", "bip72b"),
        ("memMB", "mem_mb"),
    ],
)
def test_camelized_acronyms(input_str, expected_output):
    """
    Validate decamelizing acronyms works as expected.
    :type input_str: str
    :type expected_output: str
    """
    assert humps.decamelize(input_str) == expected_output


def test_conditionals():
    assert humps.is_pascalcase("RedRobin")
    assert humps.is_snakecase("RedRobin") is False
    assert humps.is_camelcase("RedRobin") is False

    assert humps.is_snakecase("ruby_tuesdays")
    assert humps.is_camelcase("ruby_tuesdays") is False
    assert humps.is_pascalcase("jackInTheBox") is False

    assert humps.is_camelcase("jackInTheBox")
    assert humps.is_snakecase("jackInTheBox") is False
    assert humps.is_pascalcase("jackInTheBox") is False

    assert humps.is_camelcase("API")
    assert humps.is_pascalcase("API")
    assert humps.is_snakecase("API")

    # Fixed issue #128
    assert humps.is_snakecase("whatever_10")
    assert humps.is_camelcase("whatever_10") is False
    assert humps.is_pascalcase("whatever_10") is False


def test_numeric():
    assert humps.camelize(1234) == 1234
    assert humps.decamelize(123) == 123
    assert humps.pascalize(123) == 123


def test_upper():
    assert humps.camelize("API") == "API"
    assert humps.decamelize("API") == "API"
    assert humps.pascalize("API") == "API"
    assert humps.depascalize("API") == "API"


def test_pascalize():
    actual = humps.pascalize(
        {
            "videos": [
                {
                    "fallback_url": "https://media.io/video",
                    "scrubber_media_url": "https://media.io/video",
                    "dash_url": "https://media.io/video",
                }
            ],
            "images": [
                {
                    "fallback_url": "https://media.io/image",
                    "scrubber_media_url": "https://media.io/image",
                    "url": "https://media.io/image",
                }
            ],
            "other": [
                {
                    "_fallback_url": "https://media.io/image",
                    "__scrubber_media___url_": "https://media.io/image",
                    "_url__": "https://media.io/image",
                },
                {
                    "API": "test_upper",
                    "_API_": "test_upper",
                    "__API__": "test_upper",
                    "APIResponse": "test_acronym",
                    "_APIResponse_": "test_acronym",
                    "__APIResponse__": "test_acronym",
                },
            ],
        }
    )
    expected = {
        "Videos": [
            {
                "FallbackUrl": "https://media.io/video",
                "ScrubberMediaUrl": "https://media.io/video",
                "DashUrl": "https://media.io/video",
            }
        ],
        "Images": [
            {
                "FallbackUrl": "https://media.io/image",
                "ScrubberMediaUrl": "https://media.io/image",
                "Url": "https://media.io/image",
            }
        ],
        "Other": [
            {
                "_FallbackUrl": "https://media.io/image",
                "__ScrubberMediaUrl_": "https://media.io/image",
                "_Url__": "https://media.io/image",
            },
            {
                "API": "test_upper",
                "_API_": "test_upper",
                "__API__": "test_upper",
                "APIResponse": "test_acronym",
                "_APIResponse_": "test_acronym",
                "__APIResponse__": "test_acronym",
            },
        ],
    }
    assert actual == expected


def test_depascalize():
    actual = humps.depascalize(
        [
            {
                "Symbol": "AAL",
                "LastPrice": 31.78,
                "ChangePct": 2.8146,
                "ImpliedVolatality": 0.482,
            },
            {
                "Symbol": "LBTYA",
                "LastPrice": 25.95,
                "ChangePct": 2.6503,
                "ImpliedVolatality": 0.7287,
            },
            {
                "_Symbol": "LBTYK",
                "ChangePct_": 2.5827,
                "_LastPrice__": 25.42,
                "__ImpliedVolatality_": 0.4454,
            },
            {
                "API": "test_upper",
                "_API_": "test_upper",
                "__API__": "test_upper",
                "APIResponse": "test_acronym",
                "_APIResponse_": "test_acronym",
                "__APIResponse__": "test_acronym",
                "ruby_tuesdays": "ruby_tuesdays",
            },
        ]
    )
    expected = [
        {
            "symbol": "AAL",
            "last_price": 31.78,
            "change_pct": 2.8146,
            "implied_volatality": 0.482,
        },
        {
            "symbol": "LBTYA",
            "last_price": 25.95,
            "change_pct": 2.6503,
            "implied_volatality": 0.7287,
        },
        {
            "_symbol": "LBTYK",
            "change_pct_": 2.5827,
            "_last_price__": 25.42,
            "__implied_volatality_": 0.4454,
        },
        {
            "API": "test_upper",
            "_API_": "test_upper",
            "__API__": "test_upper",
            "api_response": "test_acronym",
            "_api_response_": "test_acronym",
            "__api_response__": "test_acronym",
            "ruby_tuesdays": "ruby_tuesdays",
        },
    ]

    assert actual == expected



"""
This module contains all the core logic for humps.
"""
import re

from collections.abc import Mapping

ACRONYM_RE = re.compile(r"([A-Z]+)$|([A-Z]+)(?=[A-Z0-9])")
PASCAL_RE = re.compile(r"([^\-_\s]+)")
SPLIT_RE = re.compile(r"([\-_\s]*[A-Z]+?[^A-Z\-_\s]*[\-_\s]*)")
UNDERSCORE_RE = re.compile(r"(?<=[^\-_\s])[\-_\s]+[^\-_\s]")


def pascalize(str_or_iter):
    """
    Convert a string, dict, or list of dicts to pascal case.

    :param str_or_iter:
        A string or iterable.
    :type str_or_iter: Union[list, dict, str]
    :rtype: Union[list, dict, str]
    :returns:
        pascalized string, dictionary, or list of dictionaries.
    """
    if isinstance(str_or_iter, (list, Mapping)):
        return _process_keys(str_or_iter, pascalize)

    s = str(_is_none(str_or_iter))
    if s.isupper() or s.isnumeric():
        return str_or_iter

    def _replace_fn(match):
        """
        :rtype: str
        """
        return match.group(1)[0].upper() + match.group(1)[1:]

    s = camelize(PASCAL_RE.sub(_replace_fn, s))
    return s[0].upper() + s[1:] if len(s) != 0 else s


def camelize(str_or_iter):
    """
    Convert a string, dict, or list of dicts to camel case.

    :param str_or_iter:
        A string or iterable.
    :type str_or_iter: Union[list, dict, str]
    :rtype: Union[list, dict, str]
    :returns:
        camelized string, dictionary, or list of dictionaries.
    """
    if isinstance(str_or_iter, (list, Mapping)):
        return _process_keys(str_or_iter, camelize)

    s = str(_is_none(str_or_iter))
    if s.isupper() or s.isnumeric():
        return str_or_iter

    if len(s) != 0 and not s[:2].isupper():
        s = s[0].lower() + s[1:]

    # For string "hello_world", match will contain
    #             the regex capture group for "_w".
    return UNDERSCORE_RE.sub(lambda m: m.group(0)[-1].upper(), s)


def decamelize(str_or_iter):
    """
    Convert a string, dict, or list of dicts to snake case.

    :param str_or_iter:
        A string or iterable.
    :type str_or_iter: Union[list, dict, str]
    :rtype: Union[list, dict, str]
    :returns:
        snake cased string, dictionary, or list of dictionaries.
    """
    if isinstance(str_or_iter, (list, Mapping)):
        return _process_keys(str_or_iter, decamelize)

    s = str(_is_none(str_or_iter))
    if s.isupper() or s.isnumeric():
        return str_or_iter

    return _separate_words(_fix_abbreviations(s)).lower()


def depascalize(str_or_iter):
    """
    Convert a string, dict, or list of dicts to snake case.

    :param str_or_iter: A string or iterable.
    :type str_or_iter: Union[list, dict, str]
    :rtype: Union[list, dict, str]
    :returns:
        snake cased string, dictionary, or list of dictionaries.
    """
    return decamelize(str_or_iter)


def is_camelcase(str_or_iter):
    """
    Determine if a string, dict, or list of dicts is camel case.

    :param str_or_iter:
        A string or iterable.
    :type str_or_iter: Union[list, dict, str]
    :rtype: bool
    :returns:
        True/False whether string or iterable is camel case
    """
    return str_or_iter == camelize(str_or_iter)


def is_pascalcase(str_or_iter):
    """
    Determine if a string, dict, or list of dicts is pascal case.

    :param str_or_iter: A string or iterable.
    :type str_or_iter: Union[list, dict, str]
    :rtype: bool
    :returns:
        True/False whether string or iterable is pascal case
    """
    return str_or_iter == pascalize(str_or_iter)


def is_snakecase(str_or_iter):
    """
    Determine if a string, dict, or list of dicts is snake case.

    :param str_or_iter:
        A string or iterable.
    :type str_or_iter: Union[list, dict, str]
    :rtype: bool
    :returns:
        True/False whether string or iterable is snake case
    """
    return str_or_iter == decamelize(str_or_iter)


def _is_none(_in):
    """
    Determine if the input is None.
    :param _in: input
    :return: an empty sting if _in is None, else the input is kept unchanged
    """
    return "" if _in is None else _in


def _process_keys(str_or_iter, fn):
    if isinstance(str_or_iter, list):
        return [_process_keys(k, fn) for k in str_or_iter]
    if isinstance(str_or_iter, Mapping):
        return {fn(k): _process_keys(v, fn) for k, v in str_or_iter.items()}
    return str_or_iter


def _fix_abbreviations(string):
    """
    Rewrite incorrectly cased acronyms, initialisms, and abbreviations,
    allowing them to be decamelized correctly. For example, given the string
    "APIResponse", this function is responsible for ensuring the output is
    "api_response" instead of "a_p_i_response".

    :param string: A string that may contain an incorrectly cased abbreviation.
    :type string: str
    :rtype: str
    :returns:
        A rewritten string that is safe for decamelization.
    """
    return ACRONYM_RE.sub(lambda m: m.group(0).title(), string)


def _separate_words(string, separator="_"):
    """
    Split words that are separated by case differentiation.
    :param string: Original string.
    :param separator: String by which the individual
        words will be put back together.
    :returns:
        New string.
    """
    return separator.join(s for s in SPLIT_RE.split(string) if s)



"""
Test camelization.
"""
import pytest

import humps


@pytest.mark.parametrize(
    "input_str, expected_output",
    [
        ("fallback_url", "fallbackUrl"),
        ("scrubber_media_url", "scrubberMediaUrl"),
        ("dash_url", "dashUrl"),
        ("_fallback_url", "_fallbackUrl"),
        ("__scrubber_media___url_", "__scrubberMediaUrl_"),
        ("_url__", "_url__"),
        ("API", "API"),
        ("_API_", "_API_"),
        ("__API__", "__API__"),
        ("APIResponse", "APIResponse"),
        ("_APIResponse_", "_APIResponse_"),
        ("__APIResponse__", "__APIResponse__"),
        # Fixed issue #128
        ("whatever_10", "whatever10"),
        # Fixed issue # 18
        ("test-1-2-3-4-5-6", "test123456"),
        # Fixed issue # 61
        ("test_n_test", "testNTest"),
        # Fixed issue # 148
        ("field_value_2_type", "fieldValue2Type"),
        # Fixed issue # 256
        ("", ""),
        (None, ""),
    ],
)
def test_camelize(input_str, expected_output):
    """
    :param input_str: String that will be transformed.
    :param expected_output: The expected transformation.
    """
    output = humps.camelize(input_str)
    assert output == expected_output, "{} != {}".format(
        output, expected_output
    )


def test_camelize_dict_list():
    actual = humps.camelize(
        {
            "videos": [
                {
                    "fallback_url": "https://media.io/video",
                    "scrubber_media_url": "https://media.io/video",
                    "dash_url": "https://media.io/video",
                }
            ],
            "images": [
                {
                    "fallback_url": "https://media.io/image",
                    "scrubber_media_url": "https://media.io/image",
                    "url": "https://media.io/image",
                }
            ],
            "other": [
                {
                    "_fallback_url": "https://media.io/image",
                    "__scrubber_media___url_": "https://media.io/image",
                    "_url__": "https://media.io/image",
                },
                {
                    "API": "test_upper",
                    "_API_": "test_upper",
                    "__API__": "test_upper",
                    "APIResponse": "test_acronym",
                    "_APIResponse_": "test_acronym",
                    "__APIResponse__": "test_acronym",
                },
            ],
        }
    )
    expected = {
        "videos": [
            {
                "fallbackUrl": "https://media.io/video",
                "scrubberMediaUrl": "https://media.io/video",
                "dashUrl": "https://media.io/video",
            }
        ],
        "images": [
            {
                "fallbackUrl": "https://media.io/image",
                "scrubberMediaUrl": "https://media.io/image",
                "url": "https://media.io/image",
            }
        ],
        "other": [
            {
                "_fallbackUrl": "https://media.io/image",
                "__scrubberMediaUrl_": "https://media.io/image",
                "_url__": "https://media.io/image",
            },
            {
                "API": "test_upper",
                "_API_": "test_upper",
                "__API__": "test_upper",
                "APIResponse": "test_acronym",
                "_APIResponse_": "test_acronym",
                "__APIResponse__": "test_acronym",
            },
        ],
    }
    assert actual == expected



"""
Test decamelization.
"""
import pytest

import humps


@pytest.mark.parametrize(
    "input_str, expected_output",
    [
        ("symbol", "symbol"),
        ("lastPrice", "last_price"),
        ("changePct", "change_pct"),
        ("impliedVolatility", "implied_volatility"),
        ("_symbol", "_symbol"),
        ("changePct_", "change_pct_"),
        ("_lastPrice__", "_last_price__"),
        ("__impliedVolatility_", "__implied_volatility_"),
        ("API", "API"),
        ("_API_", "_API_"),
        ("__API__", "__API__"),
        ("APIResponse", "api_response"),
        ("_APIResponse_", "_api_response_"),
        ("__APIResponse__", "__api_response__"),
        # Fixed issue #2. 2021-05-01
        ("_itemID", "_item_id"),
        # Fixed issue #4. 2021-05-01
        ("memMB", "mem_mb"),
        # Fixed issue #127. 2021-09-13
        ("sizeX", "size_x"),
        # Fixed issue #168. 2021-09-13
        ("aB", "a_b"),
        # Fixed issue #201. 2021-10-12
        ("testNTest", "test_n_test"),
    ],
)
def test_decamelize(input_str, expected_output):
    """
    :param input_str: String that will be transformed.
    :param expected_output: The expected transformation.
    """
    output = humps.decamelize(input_str)
    assert output == expected_output, "{} != {}".format(
        output, expected_output
    )


def test_decamelize_dict_list():
    actual = humps.decamelize(
        [
            {
                "symbol": "AAL",
                "lastPrice": 31.78,
                "changePct": 2.8146,
                "impliedVolatility": 0.482,
            },
            {
                "symbol": "LBTYA",
                "lastPrice": 25.95,
                "changePct": 2.6503,
                "impliedVolatility": 0.7287,
            },
            {
                "_symbol": "LBTYK",
                "changePct_": 2.5827,
                "_lastPrice__": 25.42,
                "__impliedVolatility_": 0.4454,
            },
            {
                "API": "test_upper",
                "_API_": "test_upper",
                "__API__": "test_upper",
                "APIResponse": "test_acronym",
                "_APIResponse_": "test_acronym",
                "__APIResponse__": "test_acronym",
                "ruby_tuesdays": "ruby_tuesdays",
                "_itemID": "_item_id",
            },
        ]
    )
    expected = [
        {
            "symbol": "AAL",
            "last_price": 31.78,
            "change_pct": 2.8146,
            "implied_volatility": 0.482,
        },
        {
            "symbol": "LBTYA",
            "last_price": 25.95,
            "change_pct": 2.6503,
            "implied_volatility": 0.7287,
        },
        {
            "_symbol": "LBTYK",
            "change_pct_": 2.5827,
            "_last_price__": 25.42,
            "__implied_volatility_": 0.4454,
        },
        {
            "API": "test_upper",
            "_API_": "test_upper",
            "__API__": "test_upper",
            "api_response": "test_acronym",
            "_api_response_": "test_acronym",
            "__api_response__": "test_acronym",
            "ruby_tuesdays": "ruby_tuesdays",
            "_item_id": "_item_id",
        },
    ]

    assert actual == expected, "{} != {}".format(actual, expected)



# flake8: noqa
# noreorder
"""
Underscore-to-camelCase converter (and vice versa) for strings and dict keys in Python.
"""
__title__ = "pyhumps"
__version__ = "3.6.0"
__author__ = "Nick Ficano"
__license__ = "MIT License"
__copyright__ = "Copyright 2019 Nick Ficano"

from humps.main import (
    camelize,
    decamelize,
    depascalize,
    is_camelcase,
    is_pascalcase,
    is_snakecase,
    pascalize,
)



"""
Test the utility for splitting words.
"""
import pytest

from humps.main import _separate_words


@pytest.mark.parametrize(
    "input_str, expected_output",
    [
        # Pascals.
        ("HelloWorld", "Hello_World"),
        ("_HelloWorld", "_Hello_World"),
        ("__HelloWorld", "__Hello_World"),
        ("HelloWorld_", "Hello_World_"),
        ("HelloWorld__", "Hello_World__"),
        # Camels
        ("helloWorld", "hello_World"),
        ("_helloWorld", "_hello_World"),
        ("__helloWorld", "__hello_World"),
        ("helloWorld_", "hello_World_"),
        ("helloWorld__", "hello_World__"),
        # Snakes
        ("hello_world", "hello_world"),
        ("_hello_world", "_hello_world"),
        ("__hello_world", "__hello_world"),
        ("hello_world_", "hello_world_"),
        ("hello_world__", "hello_world__"),
        # Fixes issue #128
        ("whatever_hi", "whatever_hi"),
        ("whatever_10", "whatever_10"),
        # Fixes issue #127
        ("sizeX", "size_X"),
        # Fixes issue #168
        ("aB", "a_B"),
        # Fixed issue #201. 2021-10-12
        ("testNTest", "test_N_Test"),
    ],
)
def test_separate_words(input_str, expected_output):
    """
    :param input_str: String that will be transformed.
    :param expected_output: The expected transformation.
    """
    output = _separate_words(input_str)
    assert output == expected_output, "{} != {}".format(
        output, expected_output
    )



"""This module contains setup instructions for pyhumps."""
import codecs
import os

import setuptools

here = os.path.abspath(os.path.dirname(__file__))

with codecs.open(os.path.join(here, "README.md"), encoding="utf-8") as fh:
    long_description = "\n" + fh.read()

setuptools.setup(
    name="pyhumps",
    version="3.6.0",
    author="Nick Ficano",
    author_email="nficano@gmail.com",
    packages=["humps"],
    package_data={"": ["LICENSE"], "humps": ["py.typed", "main.pyi"]},
    url="https://github.com/nficano/humps",
    license="The Unlicense (Unlicense)",
    classifiers=[
        "License :: OSI Approved :: MIT License",
        "Programming Language :: Python",
        "Programming Language :: Python :: 3",
        "Programming Language :: Python :: 3.6",
        "Programming Language :: Python :: 3.7",
        "Programming Language :: Python :: 3.8",
        "Programming Language :: Python :: 3.9",
        "Programming Language :: Python :: 3.10",
        "Programming Language :: Python :: Implementation :: CPython",
        "Programming Language :: Python :: Implementation :: PyPy",
    ],
    description=(
        "🐫  Convert strings (and dictionary keys) between snake case, camel "
        "case and pascal case in Python. Inspired by Humps for Node"
    ),
    include_package_data=True,
    long_description_content_type="text/markdown",
    long_description=long_description,
    keywords=["humps", "snakecase", "convert case", "camelcase"],
)
